@@ -1,4 +1,4 @@
-# Copyright 2014-2018 CERN for the benefit of the ATLAS collaboration.
+# Copyright 2014-2020 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,18 +14,22 @@
 #
 # Authors:
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2018
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2019
-# - Vincent Garonne <vgaronne@gmail.com>, 2014-2016
-# - Martin Barisits <martin.barisits@cern.ch>, 2014-2017
-# - Wen Guan <wguan.icedew@gmail.com>, 2014-2016
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2020
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2016
+# - Martin Barisits <martin.barisits@cern.ch>, 2014-2020
+# - Wen Guan <wen.guan@cern.ch>, 2014-2016
 # - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2016
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2016
 # - Brian Bockelman <bbockelm@cse.unl.edu>, 2018
 # - Eric Vaandering <ericvaandering@gmail.com>, 2018
+# - dciangot <diego.ciangottini@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - maatthias <maatthias@gmail.com>, 2019
 # - Gabriele Fronze' <gfronze@cern.ch>, 2019
-# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019
+# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019-2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -41,14 +45,16 @@ import logging
 import time
 import traceback
 
-from rucio.common.config import config_get
-from rucio.common.exception import InvalidRSEExpression, TransferToolTimeout, TransferToolWrongAnswer, RequestNotFound, ConfigNotFound, DuplicateFileTransferSubmission
+from rucio.common.config import config_get, config_get_bool
+from rucio.common.exception import (InvalidRSEExpression, TransferToolTimeout, TransferToolWrongAnswer, RequestNotFound,
+                                    ConfigNotFound, DuplicateFileTransferSubmission, VONotFound)
 from rucio.common.utils import chunks, set_checksum_value
 from rucio.core import request, transfer as transfer_core
 from rucio.core.config import get
 from rucio.core.monitor import record_counter, record_timer
 from rucio.core.rse import list_rses, get_rse_supported_checksums
 from rucio.core.rse_expression_parser import parse_expression
+from rucio.core.vo import list_vos
 from rucio.db.sqla.session import read_session
 from rucio.db.sqla.constants import RequestState
 from rucio.rse import rsemanager as rsemgr
@@ -440,29 +446,49 @@ def bulk_group_transfer(transfers, policy='rule', group_bulk=200, source_strateg
     return grouped_jobs
 
 
-def get_conveyor_rses(rses=None, include_rses=None, exclude_rses=None):
+def get_conveyor_rses(rses=None, include_rses=None, exclude_rses=None, vos=None):
     """
     Get a list of rses for conveyor
 
-    :param rses:          List of rses
+    :param rses:          List of rses (Single-VO only)
     :param include_rses:  RSEs to include
     :param exclude_rses:  RSEs to exclude
+    :param vos:           VOs on which to look for RSEs. Only used in multi-VO mode.
+                          If None, we either use all VOs if run from "def", or the current VO otherwise.
     :return:              List of working rses
     """
+    multi_vo = config_get_bool('common', 'multi_vo', raise_exception=False, default=False)
+    if not multi_vo:
+        if vos:
+            logging.warning('Ignoring argument vos, this is only applicable in a multi-VO setup.')
+        vos = ['def']
+    else:
+        if vos:
+            invalid = set(vos) - set([v['vo'] for v in list_vos()])
+            if invalid:
+                msg = 'VO{} {} cannot be found'.format('s' if len(invalid) > 1 else '', ', '.join([repr(v) for v in invalid]))
+                raise VONotFound(msg)
+        else:
+            vos = [v['vo'] for v in list_vos()]
+        logging.info('This instance will work on VO%s: %s' % ('s' if len(vos) > 1 else '', ', '.join([v for v in vos])))
+
     working_rses = []
-    rses_list = list_rses()
+    rses_list = []
+    for vo in vos:
+        rses_list.extend(list_rses(filters={'vo': vo}))
     if rses:
         working_rses = [rse for rse in rses_list if rse['rse'] in rses]
 
     if include_rses:
-        try:
-            parsed_rses = parse_expression(include_rses, session=None)
-        except InvalidRSEExpression as error:
-            logging.error("Invalid RSE exception %s to include RSEs", include_rses)
-        else:
-            for rse in parsed_rses:
-                if rse not in working_rses:
-                    working_rses.append(rse)
+        for vo in vos:
+            try:
+                parsed_rses = parse_expression(include_rses, filter={'vo': vo}, session=None)
+            except InvalidRSEExpression:
+                logging.error("Invalid RSE exception %s to include RSEs", include_rses)
+            else:
+                for rse in parsed_rses:
+                    if rse not in working_rses:
+                        working_rses.append(rse)
 
     if not (rses or include_rses):
         working_rses = rses_list
@@ -475,5 +501,5 @@ def get_conveyor_rses(rses=None, include_rses=None, exclude_rses=None):
         else:
             working_rses = [rse for rse in working_rses if rse not in parsed_rses]
 
-    working_rses = [rsemgr.get_rse_info(rse['rse']) for rse in working_rses]
+    working_rses = [rsemgr.get_rse_info(rse_id=rse['id']) for rse in working_rses]
     return working_rses
