@@ -14,7 +14,7 @@
 #
 # Authors:
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2018
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2019
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2020
 # - Wen Guan <wguan.icedew@gmail.com>, 2014-2015
 # - Vincent Garonne <vgaronne@gmail.com>, 2015-2018
 # - Martin Barisits <martin.barisits@cern.ch>, 2016-2017
@@ -38,6 +38,8 @@ import time
 import traceback
 
 from email.mime.text import MIMEText
+from six import PY2
+from prometheus_client import Counter
 from sqlalchemy.orm.exc import NoResultFound
 
 import stomp
@@ -60,6 +62,8 @@ logging.basicConfig(stream=sys.stdout,
 
 GRACEFUL_STOP = threading.Event()
 
+RECONNECT_COUNTER = Counter('rucio_daemons_hermes_reconnect', 'Counts Hermes reconnects to different ActiveMQ brokers', labelnames=('host',))
+
 
 def deliver_emails(once=False, send_email=True, thread=0, bulk=1000, delay=10):
     '''
@@ -99,7 +103,10 @@ def deliver_emails(once=False, send_email=True, thread=0, bulk=1000, delay=10):
                 logging.debug('[email] %i:%i - submitting: %s', heartbeat['assign_thread'],
                               heartbeat['nr_threads'], str(message))
 
-                msg = MIMEText(message['payload']['body'].encode('utf-8'))
+                if PY2:
+                    msg = MIMEText(message['payload']['body'].encode('utf-8'))
+                else:
+                    msg = MIMEText(message['payload']['body'])
 
                 msg['From'] = email_from
                 msg['To'] = ', '.join(message['payload']['to'])
@@ -250,6 +257,8 @@ def deliver_messages(once=False, brokers_resolved=None, thread=0, bulk=1000, del
                         if not conn.is_connected():
                             host_and_ports = conn.transport._Transport__host_and_ports[0][0]
                             record_counter('daemons.hermes.reconnect.%s' % host_and_ports.split('.')[0])
+                            labels = {'host': host_and_ports.split('.')[0]}
+                            RECONNECT_COUNTER.labels(**labels).inc()
                             conn.start()
                             if not use_ssl:
                                 logging.info('[broker] %i:%i - connecting with USERPASS to %s',
