@@ -13,7 +13,7 @@
 # limitations under the License.
 #
 # Authors:
-# - Vincent Garonne <vgaronne@gmail.com>, 2012-2018
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2019
 # - Angelos Molfetas <Angelos.Molfetas@cern.ch>, 2012
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2012
@@ -33,10 +33,9 @@
 
 from __future__ import print_function
 
-from os import remove, unlink, listdir, rmdir, stat, path, environ
-
-import nose.tools
 import re
+import unittest
+from os import remove, unlink, listdir, rmdir, stat, path, environ
 
 from rucio.client.accountlimitclient import AccountLimitClient
 from rucio.client.didclient import DIDClient
@@ -47,13 +46,13 @@ from rucio.common.config import config_get, config_get_bool
 from rucio.common.types import InternalScope, InternalAccount
 from rucio.common.utils import generate_uuid, get_tmp_dir, md5, render_json
 from rucio.daemons.abacus import account as abacus_account
-from rucio.tests.common import execute, account_name_generator, rse_name_generator, file_generator, scope_name_generator
 from rucio.rse import rsemanager as rsemgr
+from rucio.tests.common import execute, account_name_generator, rse_name_generator, file_generator, scope_name_generator
 
 
-class TestBinRucio():
+class TestBinRucio(unittest.TestCase):
 
-    def setup(self):
+    def setUp(self):
         if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
             self.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
             try:
@@ -91,7 +90,7 @@ class TestBinRucio():
         """CLIENT(USER): Rucio version"""
         cmd = 'bin/rucio --version'
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_true('rucio' in out or 'rucio' in err)
+        assert 'rucio' in out or 'rucio' in err
 
     def test_rucio_ping(self):
         """CLIENT(USER): Rucio ping"""
@@ -105,7 +104,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_true('Could not load Rucio configuration file' in err and err.rstrip().endswith('errconfig'))
+        assert 'Could not load Rucio configuration file' in err and err.rstrip().endswith('errconfig')
 
     def test_add_account(self):
         """CLIENT(ADMIN): Add account"""
@@ -114,7 +113,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, )
-        nose.tools.assert_equal('Added new account: %s\n' % tmp_val, out)
+        assert 'Added new account: %s\n' % tmp_val == out
 
     def test_whoami(self):
         """CLIENT(USER): Rucio whoami"""
@@ -122,19 +121,19 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, )
-        nose.tools.assert_regexp_matches(out, re.compile('.*account.*'))
+        assert 'account' in out
 
     def test_add_identity(self):
         """CLIENT(ADMIN): Add identity"""
         tmp_val = account_name_generator()
         cmd = 'rucio-admin account add %s' % tmp_val
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_equal('Added new account: %s\n' % tmp_val, out)
+        assert 'Added new account: %s\n' % tmp_val == out
         cmd = 'rucio-admin identity add --account %s --type GSS --id jdoe@CERN.CH --email jdoe@CERN.CH' % tmp_val
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, )
-        nose.tools.assert_equal('Added new identity to account: jdoe@CERN.CH-%s\n' % tmp_val, out)
+        assert 'Added new identity to account: jdoe@CERN.CH-%s\n' % tmp_val == out
 
     def test_del_identity(self):
         """CLIENT(ADMIN): Test del identity"""
@@ -151,14 +150,14 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_equal('Deleted identity: jdoe@CERN.CH\n', out)
+        assert 'Deleted identity: jdoe@CERN.CH\n' == out
         # list identities for account
         cmd = 'rucio-admin account list-identities %s' % (tmp_acc)
         print(self.marker + cmd)
         print(cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_equal('', out)
+        assert '' == out
 
     def test_attributes(self):
         """CLIENT(ADMIN): Add/List/Delete attributes"""
@@ -172,19 +171,19 @@ class TestBinRucio():
         exitcode, out, err = execute(cmd)
         print(out)
         print(err)
-        nose.tools.assert_equal(0, exitcode)
+        assert exitcode == 0
         # list attributes
         cmd = 'rucio-admin account list-attributes {0}'.format(tmp_acc)
         exitcode, out, err = execute(cmd)
         print(out)
         print(err)
-        nose.tools.assert_equal(0, exitcode)
+        assert exitcode == 0
         # delete attribute to the account
         cmd = 'rucio-admin account delete-attribute {0} --key test_attribute_key'.format(tmp_acc)
         exitcode, out, err = execute(cmd)
         print(out)
         print(err)
-        nose.tools.assert_equal(0, exitcode)
+        assert exitcode == 0
 
     def test_add_scope(self):
         """CLIENT(ADMIN): Add scope"""
@@ -196,7 +195,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_equal('Added new scope to account: %s-%s\n' % (tmp_scp, tmp_acc), out)
+        assert 'Added new scope to account: %s-%s\n' % (tmp_scp, tmp_acc) == out
 
     def test_add_rse(self):
         """CLIENT(ADMIN): Add RSE"""
@@ -205,7 +204,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, )
-        nose.tools.assert_equal('Added new deterministic RSE: %s\n' % tmp_val, out)
+        assert 'Added new deterministic RSE: %s\n' % tmp_val == out
 
     def test_add_rse_nondet(self):
         """CLIENT(ADMIN): Add non-deterministic RSE"""
@@ -214,7 +213,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, )
-        nose.tools.assert_equal('Added new non-deterministic RSE: %s\n' % tmp_val, out)
+        assert 'Added new non-deterministic RSE: %s\n' % tmp_val == out
 
     def test_list_rses(self):
         """CLIENT(ADMIN): List RSEs"""
@@ -225,7 +224,40 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, )
-        nose.tools.assert_regexp_matches(out, re.compile('.*%s.*' % tmp_val))
+        assert tmp_val in out
+
+    def test_rse_add_distance(self):
+        """CLIENT (ADMIN): Add distance to RSE"""
+        # add RSEs
+        temprse1 = rse_name_generator()
+        cmd = 'rucio-admin rse add %s' % temprse1
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert exitcode == 0
+        temprse2 = rse_name_generator()
+        cmd = 'rucio-admin rse add %s' % temprse2
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert exitcode == 0
+
+        # add distance between the RSEs
+        cmd = 'rucio-admin rse add-distance --distance 1 --ranking 1 %s %s' % (temprse1, temprse2)
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert exitcode == 0
+        cmd = 'rucio-admin rse add-distance --distance 1 --ranking 1 %s %s' % (temprse2, temprse1)
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err)
+        assert exitcode == 0
+
+        # add duplicate distance
+        print(self.marker + cmd)
+        exitcode, out, err = execute(cmd)
+        print(out, err, exitcode)
+        assert exitcode != 0
+        assert 'Distance from %s to %s already exists!' % (temprse2, temprse1) in err
 
     def test_upload(self):
         """CLIENT(USER): Upload"""
@@ -260,9 +292,9 @@ class TestBinRucio():
         upload_string_1 = (self.upload_success_str % path.basename(tmp_file1))
         upload_string_2 = (self.upload_success_str % path.basename(tmp_file2))
         upload_string_3 = (self.upload_success_str % path.basename(tmp_file3))
-        nose.tools.assert_true(upload_string_1 in out or upload_string_1 in err)
-        nose.tools.assert_true(upload_string_2 in out or upload_string_2 in err)
-        nose.tools.assert_true(upload_string_3 in out or upload_string_3 in err)
+        assert upload_string_1 in out or upload_string_1 in err
+        assert upload_string_2 in out or upload_string_2 in err
+        assert upload_string_3 in out or upload_string_3 in err
 
     def test_upload_file_register_after_upload(self):
         """CLIENT(USER): Rucio upload files with registration after upload"""
@@ -282,9 +314,9 @@ class TestBinRucio():
         upload_string_1 = (self.upload_success_str % path.basename(tmp_file1))
         upload_string_2 = (self.upload_success_str % path.basename(tmp_file2))
         upload_string_3 = (self.upload_success_str % path.basename(tmp_file3))
-        nose.tools.assert_true(upload_string_1 in out or upload_string_1 in err)
-        nose.tools.assert_true(upload_string_2 in out or upload_string_2 in err)
-        nose.tools.assert_true(upload_string_3 in out or upload_string_3 in err)
+        assert upload_string_1 in out or upload_string_1 in err
+        assert upload_string_2 in out or upload_string_2 in err
+        assert upload_string_3 in out or upload_string_3 in err
 
         # removing replica -> file on RSE should be overwritten
         # (simulating an upload error, where a part of the file is uploaded but the replica is not registered)
@@ -303,8 +335,8 @@ class TestBinRucio():
             exitcode, out, err = execute(cmd)
             print(out)
             print(err)
-            nose.tools.assert_true((self.upload_success_str % path.basename(tmp_file4)) in out)
-            nose.tools.assert_equal(checksum_tmp_file4, [replica for replica in self.replica_client.list_replicas(dids=[{'name': tmp_file1_name, 'scope': self.user}])][0]['md5'])
+            assert (self.upload_success_str % path.basename(tmp_file4)) in out
+            assert checksum_tmp_file4 == [replica for replica in self.replica_client.list_replicas(dids=[{'name': tmp_file1_name, 'scope': self.user}])][0]['md5']
 
             # try to upload file that already exists on RSE and is already registered -> no overwrite
             cmd = 'rucio -v upload --rse {0} --scope {1} --name {2} {3} --register-after-upload'.format(self.def_rse, self.user, tmp_file1_name, tmp_file4)
@@ -313,7 +345,7 @@ class TestBinRucio():
             print(out)
             print(err)
             remove(tmp_file4)
-            nose.tools.assert_true('File already registered' in out)
+            assert 'File already registered' in out
 
     def test_upload_file_guid(self):
         """CLIENT(USER): Rucio upload file with guid"""
@@ -326,7 +358,7 @@ class TestBinRucio():
         print(err)
         remove(tmp_file1)
         upload_string_1 = (self.upload_success_str % path.basename(tmp_file1))
-        nose.tools.assert_true(upload_string_1 in out or upload_string_1 in err)
+        assert upload_string_1 in out or upload_string_1 in err
 
     def test_upload_repeated_file(self):
         """CLIENT(USER): Rucio upload repeated files"""
@@ -365,7 +397,7 @@ class TestBinRucio():
         remove(tmp_file2)
         remove(tmp_file3)
         upload_string_1 = (self.upload_success_str % tmp_file1_name)
-        nose.tools.assert_true(upload_string_1 in out or upload_string_1 in err)
+        assert upload_string_1 in out or upload_string_1 in err
 
     def test_upload_repeated_file_dataset(self):
         """CLIENT(USER): Rucio upload repeated files to dataset"""
@@ -398,9 +430,9 @@ class TestBinRucio():
         print(out)
         print(err)
         # tmp_file1 must be in the dataset
-        nose.tools.assert_not_equal(re.search("{0}:{1}".format(self.user, tmp_file1_name), out), None)
+        assert re.search("{0}:{1}".format(self.user, tmp_file1_name), out) is not None
         # tmp_file3 must be in the dataset
-        nose.tools.assert_not_equal(re.search("{0}:{1}".format(self.user, tmp_file3_name), out), None)
+        assert re.search("{0}:{1}".format(self.user, tmp_file3_name), out) is not None
 
     def test_upload_file_dataset(self):
         """CLIENT(USER): Rucio upload files to dataset"""
@@ -424,7 +456,7 @@ class TestBinRucio():
         exitcode, out, err = execute(cmd)
         print(out)
         print(err)
-        nose.tools.assert_not_equal(re.search("{0}:{1}".format(self.user, tmp_file1_name), out), None)
+        assert re.search("{0}:{1}".format(self.user, tmp_file1_name), out) is not None
 
     def test_upload_file_dataset_register_after_upload(self):
         """CLIENT(USER): Rucio upload files to dataset with file registration after upload"""
@@ -448,7 +480,7 @@ class TestBinRucio():
         exitcode, out, err = execute(cmd)
         print(out)
         print(err)
-        nose.tools.assert_not_equal(re.search("{0}:{1}".format(self.user, tmp_file1_name), out), None)
+        assert re.search("{0}:{1}".format(self.user, tmp_file1_name), out) is not None
 
     def test_upload_adds_md5digest(self):
         """CLIENT(USER): Upload Checksums"""
@@ -464,8 +496,8 @@ class TestBinRucio():
         print(err)
         # When inspecting the metadata of the new file the user finds the md5 checksum
         meta = self.did_client.get_metadata(scope=self.user, name=tmp_file1_name)
-        nose.tools.assert_in('md5', meta)
-        nose.tools.assert_equal(meta['md5'], file_md5)
+        assert 'md5' in meta
+        assert meta['md5'] == file_md5
         remove(filename)
 
     def test_create_dataset(self):
@@ -475,7 +507,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_not_equal(re.search('Added ' + tmp_name, out), None)
+        assert re.search('Added ' + tmp_name, out) is not None
 
     def test_add_files_to_dataset(self):
         """CLIENT(USER): Rucio add files to dataset"""
@@ -502,7 +534,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_not_equal(re.search(tmp_file1[5:], out), None)
+        assert re.search(tmp_file1[5:], out) is not None
 
     def test_download_file(self):
         """CLIENT(USER): Rucio download files"""
@@ -522,7 +554,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_not_equal(re.search(tmp_file1[5:], out), None)
+        assert re.search(tmp_file1[5:], out) is not None
 
         tmp_file1 = file_generator()
         # add files
@@ -540,7 +572,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_not_equal(re.search(tmp_file1[5:], out), None)
+        assert re.search(tmp_file1[5:], out) is not None
 
         try:
             for i in listdir('data13_hip'):
@@ -567,7 +599,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_equal(re.search(tmp_file1[5:], out), None)
+        assert re.search(tmp_file1[5:], out) is None
         cmd = 'rucio -v download --dir /tmp {0}:{1} --filter guid={2}'.format(self.user, '*', uuid)
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
@@ -576,7 +608,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_not_equal(re.search(tmp_file1[5:], out), None)
+        assert re.search(tmp_file1[5:], out) is not None
 
         # Only use filter option to download file
         tmp_file1 = file_generator()
@@ -594,7 +626,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_equal(re.search(tmp_file1[5:], out), None)
+        assert re.search(tmp_file1[5:], out) is None
         cmd = 'rucio -v download --dir /tmp --scope {0} --filter guid={1}'.format(self.user, uuid)
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
@@ -603,7 +635,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_not_equal(re.search(tmp_file1[5:], out), None)
+        assert re.search(tmp_file1[5:], out) is not None
 
         # Only use filter option to download dataset
         tmp_file1 = file_generator()
@@ -619,7 +651,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_equal(re.search(tmp_file1[5:], out), None)
+        assert re.search(tmp_file1[5:], out) is None
         cmd = 'rucio download --dir /tmp --scope {0} --filter created_after=1900-01-01T00:00:00.000Z'.format(self.user)
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
@@ -629,7 +661,7 @@ class TestBinRucio():
         exitcode, out, err = execute(cmd)
         print(out, err)
         # TODO: https://github.com/rucio/rucio/issues/2926 !
-        # nose.tools.assert_not_equal(re.search(tmp_file1[5:], out), None)
+        # assert re.search(tmp_file1[5:], out) is not None
 
         # Use filter option to download dataset with wildcarded name
         tmp_file1 = file_generator()
@@ -646,7 +678,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_equal(re.search(tmp_file1[5:], out), None)
+        assert re.search(tmp_file1[5:], out) is None
         cmd = 'rucio download --dir /tmp {0}:{1} --filter created_after=1900-01-01T00:00:00.000Z'.format(self.user, dataset_name[0:-1] + '*')
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
@@ -655,7 +687,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_not_equal(re.search(tmp_file1[5:], out), None)
+        assert re.search(tmp_file1[5:], out) is not None
 
     def test_download_metalink_file(self):
         """CLIENT(USER): Rucio download with metalink file"""
@@ -665,12 +697,12 @@ class TestBinRucio():
         # Use filter and metalink option
         cmd = 'rucio download --scope mock --filter size=1 --metalink=test'
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_in('Arguments filter and metalink cannot be used together', err)
+        assert 'Arguments filter and metalink cannot be used together' in err
 
         # Use did and metalink option
         cmd = 'rucio download --metalink=test mock:test'
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_in('Arguments dids and metalink cannot be used together', err)
+        assert 'Arguments dids and metalink cannot be used together' in err
 
         # Download only with metalink file
         tmp_file = file_generator()
@@ -685,7 +717,7 @@ class TestBinRucio():
         remove(metalink_file_path)
         cmd = 'ls /tmp/{0}'.format(scope)
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_not_equal(re.search(tmp_file_name, out), None)
+        assert re.search(tmp_file_name, out) is not None
 
     def test_download_succeeds_md5only(self):
         """CLIENT(USER): Rucio download succeeds MD5 only"""
@@ -713,7 +745,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_not_equal(re.search(filename[5:], out), None)
+        assert re.search(filename[5:], out) is not None
         try:
             for i in listdir('data13_hip'):
                 unlink('data13_hip/%s' % i)
@@ -746,14 +778,14 @@ class TestBinRucio():
 
         report = 'Local\ checksum\:\ {0},\ Rucio\ checksum\:\ 0123456789abcdef0123456789abcdef'.format(file_md5)  # NOQA: W605
         print('searching', report, 'in', err)
-        nose.tools.assert_not_equal(re.search(report, err), None)
+        assert re.search(report, err) is not None
 
         # The file should not exist
         cmd = 'ls /tmp/'    # search in /tmp/
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_equal(re.search(filename[5:], out), None)
+        assert re.search(filename[5:], out) is None
 
         try:
             for i in listdir('data13_hip'):
@@ -787,7 +819,7 @@ class TestBinRucio():
         exitcode, out, err = execute(cmd)
         print(out, err)
         search = '{0} successfully downloaded'.format(tmp_file1[5:])  # triming '/tmp/' from filename
-        nose.tools.assert_not_equal(re.search(search, err), None)
+        assert re.search(search, err) is not None
 
     def test_create_rule(self):
         """CLIENT(USER): Rucio add rule"""
@@ -847,7 +879,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_not_equal(re.search(rule, out), None)
+        assert re.search(rule, out) is not None
 
     def test_delete_rule(self):
         """CLIENT(USER): rule deletion"""
@@ -897,7 +929,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_equal(5, len(out.splitlines()))
+        assert 5 == len(out.splitlines())
 
     def test_add_file_twice(self):
         """CLIENT(USER): Add file twice"""
@@ -909,7 +941,7 @@ class TestBinRucio():
         cmd = 'rucio upload --rse {0} --scope {1} {2}'.format(self.def_rse, self.user, tmp_file1)
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_equal(re.search("File {0}:{1} successfully uploaded on the storage".format(self.user, tmp_file1[5:]), out), None)
+        assert re.search("File {0}:{1} successfully uploaded on the storage".format(self.user, tmp_file1[5:]), out) is None
 
     def test_add_delete_add_file(self):
         """CLIENT(USER): Add/Delete/Add"""
@@ -945,7 +977,7 @@ class TestBinRucio():
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
-        nose.tools.assert_equal(re.search("File {0}:{1} successfully uploaded on the storage".format(self.user, tmp_file1[5:]), out), None)
+        assert re.search("File {0}:{1} successfully uploaded on the storage".format(self.user, tmp_file1[5:]), out) is None
 
     def test_attach_files_dataset(self):
         """CLIENT(USER): Rucio attach files to dataset"""
@@ -982,9 +1014,9 @@ class TestBinRucio():
         print(out)
         print(err)
         # tmp_file2 must be in the dataset
-        nose.tools.assert_not_equal(re.search("{0}:{1}".format(self.user, tmp_file2[5:]), out), None)
+        assert re.search("{0}:{1}".format(self.user, tmp_file2[5:]), out) is not None
         # tmp_file3 must be in the dataset
-        nose.tools.assert_not_equal(re.search("{0}:{1}".format(self.user, tmp_file3[5:]), out), None)
+        assert re.search("{0}:{1}".format(self.user, tmp_file3[5:]), out) is not None
 
     def test_detach_files_dataset(self):
         """CLIENT(USER): Rucio detach files to dataset"""
@@ -1015,9 +1047,9 @@ class TestBinRucio():
         print(out)
         print(err)
         # tmp_file1 must be in the dataset
-        nose.tools.assert_not_equal(re.search("{0}:{1}".format(self.user, tmp_file1[5:]), out), None)
+        assert re.search("{0}:{1}".format(self.user, tmp_file1[5:]), out) is not None
         # tmp_file3 must be in the dataset
-        nose.tools.assert_equal(re.search("{0}:{1}".format(self.user, tmp_file3[5:]), out), None)
+        assert re.search("{0}:{1}".format(self.user, tmp_file3[5:]), out) is None
 
     def test_attach_file_twice(self):
         """CLIENT(USER): Rucio attach a file twice"""
@@ -1037,7 +1069,7 @@ class TestBinRucio():
         exitcode, out, err = execute(cmd)
         print(out)
         print(err)
-        nose.tools.assert_not_equal(re.search("The file already exists", err), None)
+        assert re.search("The file already exists", err) is not None
 
     def test_attach_dataset_twice(self):
         """ CLIENT(USER): Rucio attach a dataset twice """
@@ -1056,7 +1088,7 @@ class TestBinRucio():
         exitcode, out, err = execute(cmd)
         print(out)
         print(err)
-        nose.tools.assert_not_equal(re.search("Data identifier already added to the destination content", err), None)
+        assert re.search("Data identifier already added to the destination content", err) is not None
 
     def test_detach_non_existing_file(self):
         """CLIENT(USER): Rucio detach a non existing file"""
@@ -1075,7 +1107,7 @@ class TestBinRucio():
         exitcode, out, err = execute(cmd)
         print(out)
         print(err)
-        nose.tools.assert_not_equal(re.search("Data identifier not found.", err), None)
+        assert re.search("Data identifier not found.", err) is not None
 
     def test_list_did_recursive(self):
         """ CLIENT(USER): List did recursive """
@@ -1098,14 +1130,14 @@ class TestBinRucio():
         # All attached DIDs are expected
         cmd = 'rucio list-dids {0}:{1} --recursive'.format(tmp_scope, tmp_container_1)
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_not_equal(re.search(tmp_container_1, out), None)
-        nose.tools.assert_not_equal(re.search(tmp_container_2, out), None)
-        nose.tools.assert_not_equal(re.search(tmp_container_3, out), None)
+        assert re.search(tmp_container_1, out) is not None
+        assert re.search(tmp_container_2, out) is not None
+        assert re.search(tmp_container_3, out) is not None
 
         # Wildcards are not allowed to use with --recursive
         cmd = 'rucio list-dids {0}:* --recursive'.format(tmp_scope)
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_not_equal(re.search("Option recursive cannot be used with wildcards", err), None)
+        assert re.search("Option recursive cannot be used with wildcards", err) is not None
 
     def test_attach_many_dids(self):
         """ CLIENT(USER): Rucio attach many (>1000) DIDs """
@@ -1127,14 +1159,14 @@ class TestBinRucio():
         print(err)
 
         # Checking if the execution was successfull and if the DIDs belong together
-        nose.tools.assert_not_equal(re.search('DIDs successfully attached', out), None)
+        assert re.search('DIDs successfully attached', out) is not None
         cmd = 'rucio list-content {0}'.format(tmp_dsn_did)
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         # first dataset must be in the container
-        nose.tools.assert_not_equal(re.search("{0}:{1}".format(self.user, files[0]['name']), out), None)
+        assert re.search("{0}:{1}".format(self.user, files[0]['name']), out) is not None
         # last dataset must be in the container
-        nose.tools.assert_not_equal(re.search("{0}:{1}".format(self.user, files[-1]['name']), out), None)
+        assert re.search("{0}:{1}".format(self.user, files[-1]['name']), out) is not None
 
         # Setup data with file
         did_file_path = 'list_dids.txt'
@@ -1156,14 +1188,14 @@ class TestBinRucio():
         remove(did_file_path)
 
         # Checking if the execution was successfull and if the DIDs belong together
-        nose.tools.assert_not_equal(re.search('DIDs successfully attached', out), None)
+        assert re.search('DIDs successfully attached', out) is not None
         cmd = 'rucio list-content {0}'.format(tmp_dsn_did)
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         # first file must be in the dataset
-        nose.tools.assert_not_equal(re.search("{0}:{1}".format(self.user, files[0]['name']), out), None)
+        assert re.search("{0}:{1}".format(self.user, files[0]['name']), out) is not None
         # last file must be in the dataset
-        nose.tools.assert_not_equal(re.search("{0}:{1}".format(self.user, files[-1]['name']), out), None)
+        assert re.search("{0}:{1}".format(self.user, files[-1]['name']), out) is not None
 
     def test_attach_many_dids_twice(self):
         """ CLIENT(USER): Attach many (>1000) DIDs twice """
@@ -1187,7 +1219,7 @@ class TestBinRucio():
         for dataset in datasets:
             cmd += ' {0}:{1}'.format(dataset['scope'], dataset['name'])
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_not_equal(re.search("DIDs successfully attached", out), None)
+        assert re.search("DIDs successfully attached", out) is not None
 
         # Attaching twice plus one DID that is not already attached
         new_dataset = {'name': 'dsn_%s' % generate_uuid(), 'scope': self.user, 'type': 'DATASET'}
@@ -1197,10 +1229,10 @@ class TestBinRucio():
         for dataset in datasets:
             cmd += ' {0}:{1}'.format(dataset['scope'], dataset['name'])
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_not_equal(re.search("DIDs successfully attached", out), None)
+        assert re.search("DIDs successfully attached", out) is not None
         cmd = 'rucio list-content {0}'.format(container)
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_not_equal(re.search("{0}:{1}".format(self.user, new_dataset['name']), out), None)
+        assert re.search("{0}:{1}".format(self.user, new_dataset['name']), out) is not None
 
     def test_import_data(self):
         """ CLIENT(ADMIN): Import data into rucio"""
@@ -1212,7 +1244,7 @@ class TestBinRucio():
             file.write(render_json(**data))
         cmd = 'rucio-admin data import {0}'.format(file_path)
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_not_equal(re.search('Data successfully imported', out), None)
+        assert re.search('Data successfully imported', out) is not None
         remove(file_path)
 
     def test_export_data(self):
@@ -1220,7 +1252,7 @@ class TestBinRucio():
         file_path = 'data_export.json'
         cmd = 'rucio-admin data export {0}'.format(file_path)
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_not_equal(re.search('Data successfully exported', out), None)
+        assert re.search('Data successfully exported', out) is not None
         remove(file_path)
 
     def test_set_tombstone(self):
@@ -1232,7 +1264,7 @@ class TestBinRucio():
         self.replica_client.add_replica(rse, scope, name, 4, 'aaaaaaaa')
         cmd = 'rucio-admin replicas set-tombstone {0}:{1} --rse {2}'.format(scope, name, rse)
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_not_equal(re.search('Set tombstone successfully', err), None)
+        assert re.search('Set tombstone successfully', err) is not None
 
         # Set tombstone on locked replica
         name = generate_uuid()
@@ -1240,13 +1272,13 @@ class TestBinRucio():
         self.rule_client.add_replication_rule([{'name': name, 'scope': scope}], 1, rse, locked=True)
         cmd = 'rucio-admin replicas set-tombstone {0}:{1} --rse {2}'.format(scope, name, rse)
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_not_equal(re.search('Replica is locked', err), None)
+        assert re.search('Replica is locked', err) is not None
 
         # Set tombstone on not found replica
         name = generate_uuid()
         cmd = 'rucio-admin replicas set-tombstone {0}:{1} --rse {2}'.format(scope, name, rse)
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_not_equal(re.search('Replica not found', err), None)
+        assert re.search('Replica not found', err) is not None
 
     def test_list_account_limits(self):
         """ CLIENT (USER): list account limits. """
@@ -1259,12 +1291,12 @@ class TestBinRucio():
         self.account_client.set_global_account_limit(account, rse_exp, global_limit)
         cmd = 'rucio list-account-limits {0}'.format(account)
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_not_equal(re.search('.*{0}.*{1}.*'.format(rse, local_limit), out), None)
-        nose.tools.assert_not_equal(re.search('.*{0}.*{1}.*'.format(rse_exp, global_limit), out), None)
+        assert re.search('.*{0}.*{1}.*'.format(rse, local_limit), out) is not None
+        assert re.search('.*{0}.*{1}.*'.format(rse_exp, global_limit), out) is not None
         cmd = 'rucio list-account-limits --rse {0} {1}'.format(rse, account)
         exitcode, out, err = execute(cmd)
-        nose.tools.assert_not_equal(re.search('.*{0}.*{1}.*'.format(rse, local_limit), out), None)
-        nose.tools.assert_not_equal(re.search('.*{0}.*{1}.*'.format(rse_exp, global_limit), out), None)
+        assert re.search('.*{0}.*{1}.*'.format(rse, local_limit), out) is not None
+        assert re.search('.*{0}.*{1}.*'.format(rse_exp, global_limit), out) is not None
         self.account_client.set_local_account_limit(account, rse, -1)
         self.account_client.set_global_account_limit(account, rse_exp, -1)
 
@@ -1294,11 +1326,11 @@ class TestBinRucio():
             abacus_account.run(once=True)
             cmd = 'rucio list-account-usage {0}'.format(account)
             exitcode, out, err = execute(cmd)
-            nose.tools.assert_not_equal(re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse, usage, local_limit, local_left), out), None)
-            nose.tools.assert_not_equal(re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse_exp, usage, global_limit, global_left), out), None)
+            assert re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse, usage, local_limit, local_left), out) is not None
+            assert re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse_exp, usage, global_limit, global_left), out) is not None
             cmd = 'rucio list-account-usage --rse {0} {1}'.format(rse, account)
             exitcode, out, err = execute(cmd)
-            nose.tools.assert_not_equal(re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse, usage, local_limit, local_left), out), None)
-            nose.tools.assert_not_equal(re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse_exp, usage, global_limit, global_left), out), None)
+            assert re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse, usage, local_limit, local_left), out) is not None
+            assert re.search('.*{0}.*{1}.*{2}.*{3}'.format(rse_exp, usage, global_limit, global_left), out) is not None
             self.account_client.set_local_account_limit(account, rse, -1)
             self.account_client.set_global_account_limit(account, rse_exp, -1)
