@@ -1,5 +1,6 @@
 #!/usr/bin/env python
-# Copyright 2012-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2018-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,27 +15,27 @@
 # limitations under the License.
 #
 # Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2018
-# - Angelos Molfetas <Angelos.Molfetas@cern.ch>, 2012
-# - Vincent Garonne <vgaronne@gmail.com>, 2012-2017
-# - Yun-Pin Sun <winter0128@gmail.com>, 2012-2013
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2013-2018
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2014
-# - Martin Barisits <martin.barisits@cern.ch>, 2017
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2018
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Ruturaj Gujar, <ruturaj.gujar23@gmail.com>, 2019
-# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019, 2020
+# - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019-2020
+# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019-2020
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - Martin Barisits <martin.barisits@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
 from __future__ import print_function
 
 import base64
+import imp
 import time
 from os.path import dirname, join
 from re import search
 from traceback import format_exc
-import imp
 
 from web import OK, BadRequest, InternalError, application, ctx, header
 from web import input as param_input
@@ -112,6 +113,7 @@ class UserPass(RucioController):
         HTTP Error:
             401 Unauthorized
 
+        :param Rucio-VO: VO name as a string (Multi-VO Only).
         :param Rucio-Account: Account identifier as a string.
         :param Rucio-Username: Username as a string.
         :param Rucio-Password: SHA1 hash of the password as a string.
@@ -130,6 +132,7 @@ class UserPass(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT')
         username = ctx.env.get('HTTP_X_RUCIO_USERNAME')
         password = ctx.env.get('HTTP_X_RUCIO_PASSWORD')
@@ -141,7 +144,7 @@ class UserPass(RucioController):
             ip = ctx.ip
 
         try:
-            result = get_auth_token_user_pass(account, username, password, appid, ip)
+            result = get_auth_token_user_pass(account, username, password, appid, ip, vo=vo)
         except AccessDenied:
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
         except RucioException as error:
@@ -202,6 +205,7 @@ class OIDC(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT', 'webui')
         auth_scope = ctx.env.get('HTTP_X_RUCIO_CLIENT_AUTHORIZE_SCOPE', "")
         audience = ctx.env.get('HTTP_X_RUCIO_CLIENT_AUTHORIZE_AUDIENCE', "")
@@ -224,7 +228,7 @@ class OIDC(RucioController):
                       'polling': polling,
                       'refresh_lifetime': refresh_lifetime,
                       'ip': ip}
-            result = get_auth_oidc(account, **kwargs)
+            result = get_auth_oidc(account, vo=vo, **kwargs)
         except AccessDenied:
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot get authentication URL from Rucio Authentication Server for account %(account)s' % locals())
         except RucioException as error:
@@ -294,23 +298,19 @@ class RedirectOIDC(RucioController):
         except AccessDenied:
             render = template.render(join(dirname(__file__), '../auth_templates/'))
             return render.auth_crash('contact')
-            raise generate_http_error(401, 'CannotAuthenticate', 'Cannot contact the Rucio Authentication Server.')
 
-        except RucioException as error:
+        except RucioException:
             render = template.render(join(dirname(__file__), '../auth_templates/'))
             return render.auth_crash('internal_error')
-            raise generate_http_error(500, error.__class__.__name__, error.args[0])
 
-        except Exception as error:
+        except Exception:
             print(format_exc())
             render = template.render(join(dirname(__file__), '../auth_templates/'))
             return render.auth_crash('internal_error')
-            raise InternalError(error)
 
         if not result:
             render = template.render(join(dirname(__file__), '../auth_templates/'))
             return render.auth_crash('no_token')
-            raise generate_http_error(401, 'CannotAuthenticate', 'Cannot contact the Rucio Authentication Server.')
         if fetchtoken:
             # this is only a case of returning the final token to the Rucio Client polling
             # or requesting token after copy-pasting the Rucio code from the web page page
@@ -323,7 +323,7 @@ class RedirectOIDC(RucioController):
 
 class CodeOIDC(RucioController):
     """
-    IdP redirects to this endpoing with the AuthZ code
+    IdP redirects to this endpoint with the AuthZ code
     Rucio Auth server will request new token. This endpoint should be reached
     only if the request/ IdP login has been made through web browser. Then the response
     content will be in html (including the potential errors displayed).
@@ -381,21 +381,17 @@ class CodeOIDC(RucioController):
         except AccessDenied:
             render = template.render(join(dirname(__file__), '../auth_templates/'))
             return render.auth_crash('contact')
-            raise generate_http_error(401, 'CannotAuthorize', 'Cannot authorize token request.')
-        except RucioException as error:
+        except RucioException:
             render = template.render(join(dirname(__file__), '../auth_templates/'))
             return render.auth_crash('internal_error')
-            raise generate_http_error(500, error.__class__.__name__, error.args[0])
-        except Exception as error:
+        except Exception:
             print(format_exc())
             render = template.render(join(dirname(__file__), '../auth_templates/'))
             return render.auth_crash('internal_error')
-            raise InternalError(error)
 
         render = template.render(join(dirname(__file__), '../auth_templates/'))
         if not result:
             return render.auth_crash('no_result')
-            raise generate_http_error(401, 'CannotAuthorize', 'Cannot authorize token request.')
         if 'fetchcode' in result:
             authcode = result['fetchcode']
             return render.auth_granted(authcode)
@@ -404,7 +400,6 @@ class CodeOIDC(RucioController):
             return render.auth_granted(authcode)
         else:
             return render.auth_crash('bad_request')
-            raise BadRequest()
 
 
 class TokenOIDC(RucioController):
@@ -537,11 +532,12 @@ class RefreshOIDC(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT')
         token = ctx.env.get('HTTP_X_RUCIO_AUTH_TOKEN')
 
         try:
-            result = refresh_cli_auth_token(token, account)
+            result = refresh_cli_auth_token(token, account, vo=vo)
 
         except AccessDenied:
             raise generate_http_error(401, 'CannotAuthorize', 'Cannot authorize token request.')
@@ -594,6 +590,7 @@ class GSS(RucioController):
         HTTP Error:
             401 Unauthorized
 
+        :param Rucio-VO: VO name as a string (Multi-VO only).
         :param Rucio-Account: Account identifier as a string.
         :param Rucio-AppID: Application identifier as a string.
         :param SavedCredentials: Apache mod_auth_kerb SavedCredentials.
@@ -611,6 +608,7 @@ class GSS(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT')
         gsscred = ctx.env.get('REMOTE_USER')
         appid = ctx.env.get('HTTP_X_RUCIO_APPID')
@@ -621,18 +619,16 @@ class GSS(RucioController):
             ip = ctx.ip
 
         try:
-            result = get_auth_token_gss(account, gsscred, appid, ip)
+            result = get_auth_token_gss(account, gsscred, appid, ip, vo=vo)
         except AccessDenied:
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
 
         if result is None:
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
-        else:
-            header('X-Rucio-Auth-Token', result.token)
-            header('X-Rucio-Auth-Token-Expires', date_to_str(result.expired_at))
-            return str()
 
-        raise BadRequest()
+        header('X-Rucio-Auth-Token', result.token)
+        header('X-Rucio-Auth-Token-Expires', date_to_str(result.expired_at))
+        return str()
 
 
 class x509(RucioController):
@@ -664,6 +660,7 @@ class x509(RucioController):
         HTTP Error:
             401 Unauthorized
 
+        :param Rucio-VO: VO name as a string (Multi-VO only).
         :param Rucio-Account: Account identifier as a string.
         :param Rucio-AppID: Application identifier as a string.
         :param SSLStdEnv: Apache mod_ssl SSL Standard Env Variables.
@@ -681,6 +678,7 @@ class x509(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT')
         dn = ctx.env.get('SSL_CLIENT_S_DN')
         if not dn:
@@ -711,16 +709,16 @@ class x509(RucioController):
                 break
 
         try:
-            result = get_auth_token_x509(account, dn, appid, ip)
+            result = get_auth_token_x509(account, dn, appid, ip, vo=vo)
         except AccessDenied:
-            print('Cannot Authenticate', account, dn, appid, ip)
+            print('Cannot Authenticate', account, dn, appid, ip, vo)
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
         except IdentityError:
-            print('Cannot Authenticate', account, dn, appid, ip)
+            print('Cannot Authenticate', account, dn, appid, ip, vo)
             raise generate_http_error(401, 'CannotAuthenticate', 'No default account set for %(dn)s' % locals())
 
         if not result:
-            print('Cannot Authenticate', account, dn, appid, ip)
+            print('Cannot Authenticate', account, dn, appid, ip, vo)
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
 
         header('X-Rucio-Auth-Token', result.token)
@@ -757,6 +755,7 @@ class SSH(RucioController):
         HTTP Error:
             401 Unauthorized
 
+        :param Rucio-VO: VO name as a string (Multi-VO only).
         :param Rucio-Account: Account identifier as a string.
         :param Rucio-SSH-Signature: Response to server challenge signed with SSH private key as a base64 encoded string.
         :param Rucio-AppID: Application identifier as a string.
@@ -774,6 +773,7 @@ class SSH(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT')
         signature = ctx.env.get('HTTP_X_RUCIO_SSH_SIGNATURE')
         appid = ctx.env.get('HTTP_X_RUCIO_APPID')
@@ -790,7 +790,7 @@ class SSH(RucioController):
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with malformed signature' % locals())
 
         try:
-            result = get_auth_token_ssh(account, signature, appid, ip)
+            result = get_auth_token_ssh(account, signature, appid, ip, vo=vo)
         except AccessDenied:
             raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
         except RucioException as error:
@@ -836,6 +836,7 @@ class SSHChallengeToken(RucioController):
         HTTP Error:
             401 Unauthorized
 
+        :param Rucio-VO: VO name as a string (Multi-VO only).
         :param Rucio-Account: Account identifier as a string.
         :param Rucio-AppID: Application identifier as a string.
         :returns: "Rucio-Auth-Token" as a variable-length string header.
@@ -852,6 +853,7 @@ class SSHChallengeToken(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT')
         appid = ctx.env.get('HTTP_X_RUCIO_APPID')
         if appid is None:
@@ -861,7 +863,7 @@ class SSHChallengeToken(RucioController):
             ip = ctx.ip
 
         try:
-            result = get_ssh_challenge_token(account, appid, ip)
+            result = get_ssh_challenge_token(account, appid, ip, vo=vo)
         except RucioException as error:
             raise generate_http_error(500, error.__class__.__name__, error.args[0])
         except Exception as error:
@@ -905,6 +907,7 @@ class SAML(RucioController):
         HTTP Error:
             401 Unauthorized
 
+        :param Rucio-VO: VO name as a string (Multi-VO only)
         :param Rucio-Account: Account identifier as a string.
         :param Rucio-Username: Username as a string.
         :param Rucio-Password: Password as a string.
@@ -923,7 +926,12 @@ class SAML(RucioController):
         header('Cache-Control', 'post-check=0, pre-check=0', False)
         header('Pragma', 'no-cache')
 
+        if not EXTRA_MODULES['onelogin']:
+            header('X-Rucio-Auth-Token', None)
+            return "SAML not configured on the server side."
+
         saml_nameid = cookies().get('saml-nameid')
+        vo = ctx.env.get('HTTP_X_RUCIO_VO', 'def')
         account = ctx.env.get('HTTP_X_RUCIO_ACCOUNT')
         appid = ctx.env.get('HTTP_X_RUCIO_APPID')
         if appid is None:
@@ -934,7 +942,7 @@ class SAML(RucioController):
 
         if saml_nameid:
             try:
-                result = get_auth_token_saml(account, saml_nameid, appid, ip)
+                result = get_auth_token_saml(account, saml_nameid, appid, ip, vo=vo)
             except AccessDenied:
                 raise generate_http_error(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
             except RucioException as error:
@@ -962,6 +970,11 @@ class SAML(RucioController):
         return str()
 
     def POST(self):
+
+        if not EXTRA_MODULES['onelogin']:
+            header('X-Rucio-Auth-Token', None)
+            return "SAML not configured on the server side."
+
         SAML_PATH = config_get('saml', 'config_path')
         request = ctx.env
         data = dict(param_input())
