@@ -1,5 +1,6 @@
 #!/usr/bin/env python
-# Copyright 2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2018-2020 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,28 +15,29 @@
 # limitations under the License.
 #
 # Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2018
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2014
-# - Yun-Pin Sun <yun-pin.sun@cern.ch>, 2012
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2014
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2018
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
+# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2019
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
 from __future__ import print_function
+
 import base64
 from re import search
 from traceback import format_exc
 
-from rucio.api.authentication import get_auth_token_user_pass, get_auth_token_gss, get_auth_token_x509, get_auth_token_ssh, get_ssh_challenge_token, validate_auth_token
-from rucio.common.exception import AccessDenied, IdentityError, RucioException
-from rucio.common.utils import generate_http_error_flask
-from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask
-
 from flask import Flask, Blueprint, request, Response
 from flask.views import MethodView
 
+from rucio.api.authentication import get_auth_token_user_pass, get_auth_token_gss, get_auth_token_x509, get_auth_token_ssh, get_ssh_challenge_token, validate_auth_token
+from rucio.common.exception import AccessDenied, IdentityError, RucioException
+from rucio.common.utils import generate_http_error_flask, date_to_str
+from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask
+
 
 class UserPass(MethodView):
     """
@@ -69,6 +71,7 @@ class UserPass(MethodView):
 
         .. :quickref: UserPass; Authenticate with username/password
 
+        :reqheader X-Rucio-VO: VO name as a string (Multi-VO Only)
         :reqheader X-Rucio-Account: Account identifier as a string.
         :reqheader X-Rucio-Username: Username as a string.
         :reqheader X-Rucio-Password: password as a text-plain string.
@@ -95,6 +98,7 @@ class UserPass(MethodView):
         response.headers['Cache-Control'] = 'post-check=0, pre-check=0'
         response.headers['Pragma'] = 'no-cache'
 
+        vo = request.environ.get('HTTP_X_RUCIO_VO', 'def')
         account = request.environ.get('HTTP_X_RUCIO_ACCOUNT')
         username = request.environ.get('HTTP_X_RUCIO_USERNAME')
         password = request.environ.get('HTTP_X_RUCIO_PASSWORD')
@@ -107,7 +111,7 @@ class UserPass(MethodView):
 
         print(account, username, password, appid)
         try:
-            result = get_auth_token_user_pass(account, username, password, appid, ip)
+            result = get_auth_token_user_pass(account, username, password, appid, ip, vo=vo)
         except AccessDenied:
             return generate_http_error_flask(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
         except RucioException as error:
@@ -119,7 +123,8 @@ class UserPass(MethodView):
         if not result:
             return generate_http_error_flask(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
 
-        response.headers['X-Rucio-Auth-Token'] = result
+        response.headers['X-Rucio-Auth-Token'] = result.token
+        response.headers['X-Rucio-Auth-Token-Expires'] = date_to_str(result.expired_at)
         return response
 
 
@@ -151,6 +156,7 @@ class GSS(MethodView):
 
         .. :quickref: GSS; Authenticate with GSS token
 
+        :reqheader Rucio-VO: VO name as a string (Multi-VO only).
         :reqheader Rucio-Account: Account identifier as a string.
         :reqheader Rucio-AppID: Application identifier as a string.
         :reqheader SavedCredentials: Apache mod_auth_kerb SavedCredentials.
@@ -176,6 +182,7 @@ class GSS(MethodView):
         response.headers['Cache-Control'] = 'post-check=0, pre-check=0'
         response.headers['Pragma'] = 'no-cache'
 
+        vo = request.environ.get('HTTP_X_RUCIO_VO', 'def')
         account = request.environ.get('HTTP_X_RUCIO_ACCOUNT')
         gsscred = request.environ.get('REMOTE_USER')
         appid = request.environ.get('HTTP_X_RUCIO_APPID')
@@ -186,17 +193,16 @@ class GSS(MethodView):
             ip = request.remote_addr
 
         try:
-            result = get_auth_token_gss(account, gsscred, appid, ip)
+            result = get_auth_token_gss(account, gsscred, appid, ip, vo=vo)
         except AccessDenied:
             return generate_http_error_flask(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
 
         if result is None:
             return generate_http_error_flask(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
-        else:
-            response.headers('X-Rucio-Auth-Token', result)
-            return str()
 
-        return 'BadRequest', 400
+        response.headers['X-Rucio-Auth-Token'] = result.token
+        response.headers['X-Rucio-Auth-Token-Expires'] = date_to_str(result.expired_at)
+        return response
 
 
 class x509(MethodView):
@@ -227,6 +233,7 @@ class x509(MethodView):
 
         .. :quickref: x509; Authenticate with x509 certificate.
 
+        :reqheader Rucio-VO: VO name as a string (Multi-VO only).
         :reqheader Rucio-Account: Account identifier as a string.
         :reqheader Rucio-AppID: Application identifier as a string.
         :reqheader SSLStdEnv: Apache mod_ssl SSL Standard Env Variables.
@@ -252,6 +259,7 @@ class x509(MethodView):
         response.headers['Cache-Control'] = 'post-check=0, pre-check=0'
         response.headers['Pragma'] = 'no-cache'
 
+        vo = request.environ.get('HTTP_X_RUCIO_VO', 'def')
         account = request.environ.get('HTTP_X_RUCIO_ACCOUNT')
         dn = request.environ.get('SSL_CLIENT_S_DN')
         if not dn:
@@ -282,20 +290,20 @@ class x509(MethodView):
                 break
 
         try:
-            result = get_auth_token_x509(account, dn, appid, ip)
+            result = get_auth_token_x509(account, dn, appid, ip, vo=vo)
         except AccessDenied:
-            print('Cannot Authenticate', account, dn, appid, ip)
+            print('Cannot Authenticate', account, dn, appid, ip, vo)
             return generate_http_error_flask(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
         except IdentityError:
-            print('Cannot Authenticate', account, dn, appid, ip)
+            print('Cannot Authenticate', account, dn, appid, ip, vo)
             return generate_http_error_flask(401, 'CannotAuthenticate', 'No default account set for %(dn)s' % locals())
 
         if not result:
-            print('Cannot Authenticate', account, dn, appid, ip)
+            print('Cannot Authenticate', account, dn, appid, ip, vo)
             return generate_http_error_flask(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
 
-        response.headers['X-Rucio-Auth-Token'] = result
-        response.set_data(str())
+        response.headers['X-Rucio-Auth-Token'] = result.token
+        response.headers['X-Rucio-Auth-Token-Expires'] = date_to_str(result.expired_at)
         return response
 
 
@@ -327,6 +335,7 @@ class SSH(MethodView):
 
         .. :quickref: SSH; Authenticate with SSH key exchange.
 
+        :reqheader Rucio-VO: VO name as a string (Multi-VO only).
         :reqheader Rucio-Account: Account identifier as a string.
         :reqheader Rucio-SSH-Signature: Response to server challenge signed with SSH private key as a base64 encoded string.
         :reqheader Rucio-AppID: Application identifier as a string.
@@ -352,6 +361,7 @@ class SSH(MethodView):
         response.headers['Cache-Control'] = 'post-check=0, pre-check=0'
         response.headers['Pragma'] = 'no-cache'
 
+        vo = request.environ.get('HTTP_X_RUCIO_VO', 'def')
         account = request.environ.get('HTTP_X_RUCIO_ACCOUNT')
         signature = request.environ.get('HTTP_X_RUCIO_SSH_SIGNATURE')
         appid = request.environ.get('HTTP_X_RUCIO_APPID')
@@ -368,7 +378,7 @@ class SSH(MethodView):
             return generate_http_error_flask(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with malformed signature' % locals())
 
         try:
-            result = get_auth_token_ssh(account, signature, appid, ip)
+            result = get_auth_token_ssh(account, signature, appid, ip, vo=vo)
         except AccessDenied:
             return generate_http_error_flask(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
         except RucioException as error:
@@ -380,8 +390,8 @@ class SSH(MethodView):
         if not result:
             return generate_http_error_flask(401, 'CannotAuthenticate', 'Cannot authenticate to account %(account)s with given credentials' % locals())
 
-        response.headers['X-Rucio-Auth-Token'] = result
-        response.set_data(str())
+        response.headers['X-Rucio-Auth-Token'] = result.token
+        response.headers['X-Rucio-Auth-Token-Expires'] = date_to_str(result.expired_at)
         return response
 
 
@@ -413,6 +423,7 @@ class SSHChallengeToken(MethodView):
 
         .. :quickref: SSHChallengeToken; Request SSH Challenge Token
 
+        :reqheader Rucio-VO: VO name as a string (Multi-VO only).
         :reqheader Rucio-Account: Account identifier as a string.
         :reqheader Rucio-AppID: Application identifier as a string.
         :resheader Access-Control-Allow-Origin:
@@ -437,6 +448,7 @@ class SSHChallengeToken(MethodView):
         response.headers['Cache-Control'] = 'post-check=0, pre-check=0'
         response.headers['Pragma'] = 'no-cache'
 
+        vo = request.environ.get('HTTP_X_RUCIO_VO', 'def')
         account = request.environ.get('HTTP_X_RUCIO_ACCOUNT')
         appid = request.environ.get('HTTP_X_RUCIO_APPID')
         if appid is None:
@@ -446,7 +458,7 @@ class SSHChallengeToken(MethodView):
             ip = request.remote_addr
 
         try:
-            result = get_ssh_challenge_token(account, appid, ip)
+            result = get_ssh_challenge_token(account, appid, ip, vo=vo)
         except RucioException as error:
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
         except Exception as error:
@@ -456,8 +468,8 @@ class SSHChallengeToken(MethodView):
         if not result:
             return generate_http_error_flask(401, 'CannotAuthenticate', 'Cannot generate challenge for account %(account)s' % locals())
 
-        response.headers['X-Rucio-SSH-Challenge-Token'] = result
-        response.set_data(str())
+        response.headers['X-Rucio-Auth-Token'] = result.token
+        response.headers['X-Rucio-Auth-Token-Expires'] = date_to_str(result.expired_at)
         return response
 
 
