@@ -21,6 +21,8 @@
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2017
 # - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -74,7 +76,7 @@ class Rule(MethodView):
         except ValueError:
             estimate_ttc = False
         try:
-            rule = get_replication_rule(rule_id, estimate_ttc=estimate_ttc)
+            rule = get_replication_rule(rule_id, estimate_ttc=estimate_ttc, issuer=request.environ.get('issuer'), vo=request.environ.get('vo'))
         except RuleNotFound as error:
             return generate_http_error_flask(404, 'RuleNotFound', error.args[0])
         except RucioException as error:
@@ -98,7 +100,7 @@ class Rule(MethodView):
         try:
             params = loads(json_data)
             options = params['options']
-            update_replication_rule(rule_id=rule_id, options=options, issuer=request.environ.get('issuer'))
+            update_replication_rule(rule_id=rule_id, options=options, issuer=request.environ.get('issuer'), vo=request.environ.get('vo'))
         except AccessDenied as error:
             return generate_http_error_flask(401, 'AccessDenied', error.args[0])
         except RuleNotFound as error:
@@ -135,7 +137,7 @@ class Rule(MethodView):
             return generate_http_error_flask(400, 'ValueError', 'Cannot decode json parameter list')
 
         try:
-            delete_replication_rule(rule_id=rule_id, purge_replicas=purge_replicas, issuer=request.environ.get('issuer'))
+            delete_replication_rule(rule_id=rule_id, purge_replicas=purge_replicas, issuer=request.environ.get('issuer'), vo=request.environ.get('vo'))
         except AccessDenied as error:
             return generate_http_error_flask(401, 'AccessDenied', error.args[0])
         except UnsupportedOperation as error:
@@ -171,7 +173,7 @@ class AllRule(MethodView):
 
         try:
             data = ""
-            for rule in list_replication_rules(filters=filters):
+            for rule in list_replication_rules(filters=filters, vo=request.environ.get('vo')):
                 data += dumps(rule, cls=APIEncoder) + '\n'
             return Response(data, content_type="application/x-json-stream")
         except RuleNotFound as error:
@@ -293,7 +295,8 @@ class AllRule(MethodView):
                                             priority=priority,
                                             split_container=split_container,
                                             meta=meta,
-                                            issuer=request.environ.get('issuer'))
+                                            issuer=request.environ.get('issuer'),
+                                            vo=request.environ.get('vo'))
         # TODO: Add all other error cases here
         except InvalidReplicationRule as error:
             return generate_http_error_flask(409, 'InvalidReplicationRule', error.args[0])
@@ -387,7 +390,8 @@ class ReduceRule(MethodView):
             rule_ids = reduce_replication_rule(rule_id=rule_id,
                                                copies=copies,
                                                exclude_expression=exclude_expression,
-                                               issuer=request.environ.get('issuer'))
+                                               issuer=request.environ.get('issuer'),
+                                               vo=request.environ.get('vo'))
         # TODO: Add all other error cases here
         except RuleReplaceFailed as error:
             return generate_http_error_flask(409, 'RuleReplaceFailed', error.args[0])
@@ -429,7 +433,8 @@ class MoveRule(MethodView):
         try:
             rule_ids = move_replication_rule(rule_id=rule_id,
                                              rse_expression=rse_expression,
-                                             issuer=request.environ.get('issuer'))
+                                             issuer=request.environ.get('issuer'),
+                                             vo=request.environ.get('vo'))
         except RuleReplaceFailed as error:
             return generate_http_error_flask(409, 'RuleReplaceFailed', error.args[0])
         except RuleNotFound as error:
@@ -460,7 +465,7 @@ class RuleHistory(MethodView):
         :returns: JSON dict containing informations about the requested user.
         """
         try:
-            history = list_replication_rule_history(rule_id)
+            history = list_replication_rule_history(rule_id, issuer=request.environ.get('issuer'), vo=request.environ.get('vo'))
         except RucioException as error:
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
         except Exception as error:
@@ -488,7 +493,7 @@ class RuleHistoryFull(MethodView):
         :returns: JSON dict containing informations about the requested user.
         """
         try:
-            history = list_replication_rule_full_history(scope, name)
+            history = list_replication_rule_full_history(scope, name, vo=request.environ.get('vo'))
         except RucioException as error:
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
         except Exception as error:
@@ -516,7 +521,7 @@ class RuleAnalysis(MethodView):
         :returns: JSON dict containing informations about the requested user.
         """
         try:
-            analysis = examine_replication_rule(rule_id)
+            analysis = examine_replication_rule(rule_id, issuer=request.environ.get('issuer'), vo=request.environ.get('vo'))
         except RucioException as error:
             return generate_http_error_flask(500, error.__class__.__name__, error.args[0])
         except Exception as error:
