@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# Copyright 2012-2018 CERN for the benefit of the ATLAS collaboration.
+# Copyright 2012-2020 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,10 +18,12 @@
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2019
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2013
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2019
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2018
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2020
 # - Martin Barisits <martin.barisits@cern.ch>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Luc Goossens <luc.goossens@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -30,6 +32,7 @@ from datetime import datetime
 from json import dumps, loads
 from six import string_types
 from traceback import format_exc
+
 try:
     from urllib import unquote
     from urlparse import parse_qs
@@ -41,7 +44,7 @@ from xml.sax.saxutils import escape
 
 from geoip2.errors import AddressNotFoundError
 
-from rucio.api.replica import (add_replicas, list_replicas, list_dataset_replicas,
+from rucio.api.replica import (add_replicas, list_replicas, list_dataset_replicas, list_dataset_replicas_bulk,
                                delete_replicas, list_dataset_replicas_vp,
                                get_did_from_pfns, update_replicas_states,
                                declare_bad_file_replicas, add_bad_pfns, get_suspicious_files,
@@ -53,9 +56,9 @@ from rucio.common.config import config_get
 from rucio.common.exception import (AccessDenied, DataIdentifierAlreadyExists, InvalidType,
                                     DataIdentifierNotFound, Duplicate, InvalidPath,
                                     ResourceTemporaryUnavailable, RucioException,
-                                    RSENotFound, UnsupportedOperation, ReplicaNotFound)
+                                    RSENotFound, UnsupportedOperation, ReplicaNotFound, InvalidObject)
 from rucio.common.replica_sorter import sort_random, sort_geoip, sort_closeness, sort_dynamic, sort_ranking
-from rucio.common.schema import SCOPE_NAME_REGEXP
+from rucio.common.schema import get_schema_value
 from rucio.common.utils import generate_http_error, parse_response, APIEncoder, render_json_list
 from rucio.common.constants import SUPPORTED_PROTOCOLS
 from rucio.web.rest.common import rucio_loadhook, rucio_unloadhook, RucioController, check_accept_header_wrapper
@@ -69,9 +72,10 @@ URLS = ('/list/?$', 'ListReplicas',
         '/rse/(.*)/?$', 'ReplicasRSE',
         '/bad/?$', 'BadReplicas',
         '/dids/?$', 'ReplicasDIDs',
-        '%s/datasets$' % SCOPE_NAME_REGEXP, 'DatasetReplicas',
-        '%s/datasets_vp$' % SCOPE_NAME_REGEXP, 'DatasetReplicasVP',
-        '%s/?$' % SCOPE_NAME_REGEXP, 'Replicas',
+        '%s/datasets$' % get_schema_value('SCOPE_NAME_REGEXP'), 'DatasetReplicas',
+        '/datasets_bulk/?$', 'DatasetReplicasBulk',
+        '%s/datasets_vp$' % get_schema_value('SCOPE_NAME_REGEXP'), 'DatasetReplicasVP',
+        '%s/?$' % get_schema_value('SCOPE_NAME_REGEXP'), 'Replicas',
         '/tombstone/?$', 'Tombstone')
 
 
@@ -121,7 +125,7 @@ class Replicas(RucioController):
             __first = True
 
             # then, stream the replica information
-            for rfile in list_replicas(dids=dids, schemes=schemes):
+            for rfile in list_replicas(dids=dids, schemes=schemes, vo=ctx.env.get('vo')):
 
                 # in first round, set the appropriate content type, and stream the header
                 if __first:
@@ -206,8 +210,9 @@ class Replicas(RucioController):
             raise generate_http_error(400, 'ValueError', 'Cannot decode json parameter list')
 
         try:
-            add_replicas(rse=parameters['rse'], files=parameters['files'], issuer=ctx.env.get(
-                'issuer'), ignore_availability=parameters.get('ignore_availability', False))
+            add_replicas(rse=parameters['rse'], files=parameters['files'],
+                         issuer=ctx.env.get('issuer'), vo=ctx.env.get('vo'),
+                         ignore_availability=parameters.get('ignore_availability', False))
         except InvalidPath as error:
             raise generate_http_error(400, 'InvalidPath', error.args[0])
         except AccessDenied as error:
@@ -245,7 +250,7 @@ class Replicas(RucioController):
             raise generate_http_error(400, 'ValueError', 'Cannot decode json parameter list')
 
         try:
-            update_replicas_states(rse=parameters['rse'], files=parameters['files'], issuer=ctx.env.get('issuer'))
+            update_replicas_states(rse=parameters['rse'], files=parameters['files'], issuer=ctx.env.get('issuer'), vo=ctx.env.get('vo'))
         except AccessDenied as error:
             raise generate_http_error(401, 'AccessDenied', error.args[0])
         except UnsupportedOperation as error:
@@ -276,8 +281,9 @@ class Replicas(RucioController):
             raise generate_http_error(400, 'ValueError', 'Cannot decode json parameter list')
 
         try:
-            delete_replicas(rse=parameters['rse'], files=parameters['files'], issuer=ctx.env.get(
-                'issuer'), ignore_availability=parameters.get('ignore_availability', False))
+            delete_replicas(rse=parameters['rse'], files=parameters['files'],
+                            issuer=ctx.env.get('issuer'), vo=ctx.env.get('vo'),
+                            ignore_availability=parameters.get('ignore_availability', False))
         except AccessDenied as error:
             raise generate_http_error(401, 'AccessDenied', error.args[0])
         except RSENotFound as error:
@@ -402,7 +408,8 @@ class ListReplicas(RucioController):
                                        resolve_archives=resolve_archives,
                                        resolve_parents=resolve_parents,
                                        updated_after=updated_after,
-                                       issuer=ctx.env.get('issuer')):
+                                       issuer=ctx.env.get('issuer'),
+                                       vo=ctx.env.get('vo')):
 
                 # in first round, set the appropriate content type, and stream the header
                 if __first:
@@ -519,7 +526,7 @@ class ReplicasDIDs(RucioController):
             raise generate_http_error(400, 'ValueError', 'Cannot decode json parameter list')
 
         try:
-            for pfn in get_did_from_pfns(pfns, rse):
+            for pfn in get_did_from_pfns(pfns, rse, vo=ctx.env.get('vo')):
                 yield dumps(pfn) + '\n'
         except AccessDenied as error:
             raise generate_http_error(401, 'AccessDenied', error.args[0])
@@ -561,7 +568,7 @@ class BadReplicas(RucioController):
 
         not_declared_files = {}
         try:
-            not_declared_files = declare_bad_file_replicas(pfns=pfns, reason=reason, issuer=ctx.env.get('issuer'))
+            not_declared_files = declare_bad_file_replicas(pfns=pfns, reason=reason, issuer=ctx.env.get('issuer'), vo=ctx.env.get('vo'))
         except AccessDenied as error:
             raise generate_http_error(401, 'AccessDenied', error.args[0])
         except ReplicaNotFound as error:
@@ -604,7 +611,7 @@ class SuspiciousReplicas(RucioController):
 
         not_declared_files = {}
         try:
-            not_declared_files = declare_suspicious_file_replicas(pfns=pfns, reason=reason, issuer=ctx.env.get('issuer'))
+            not_declared_files = declare_suspicious_file_replicas(pfns=pfns, reason=reason, issuer=ctx.env.get('issuer'), vo=ctx.env.get('vo'))
         except AccessDenied as error:
             raise generate_http_error(401, 'AccessDenied', error.args[0])
         except RucioException as error:
@@ -644,7 +651,7 @@ class SuspiciousReplicas(RucioController):
                 nattempts = int(params['nattempts'][0])
 
         try:
-            result = get_suspicious_files(rse_expression=rse_expression, younger_than=younger_than, nattempts=nattempts)
+            result = get_suspicious_files(rse_expression=rse_expression, younger_than=younger_than, nattempts=nattempts, vo=ctx.env.get('vo'))
         except RucioException as error:
             raise generate_http_error(500, error.__class__.__name__, error.args[0])
         except Exception as error:
@@ -693,7 +700,8 @@ class BadReplicasStates(RucioController):
 
         try:
             result = list_bad_replicas_status(state=state, rse=rse, younger_than=younger_than,
-                                              older_than=older_than, limit=limit, list_pfns=list_pfns)
+                                              older_than=older_than, limit=limit, list_pfns=list_pfns,
+                                              vo=ctx.env.get('vo'))
         except RucioException as error:
             raise generate_http_error(500, error.__class__.__name__, error.args[0])
         except Exception as error:
@@ -734,7 +742,7 @@ class BadReplicasSummary(RucioController):
                 to_date = datetime.strptime(params['to_date'][0], "%Y-%m-%d")
 
         try:
-            result = get_bad_replicas_summary(rse_expression=rse_expression, from_date=from_date, to_date=to_date)
+            result = get_bad_replicas_summary(rse_expression=rse_expression, from_date=from_date, to_date=to_date, vo=ctx.env.get('vo'))
         except RucioException as error:
             raise generate_http_error(500, error.__class__.__name__, error.args[0])
         except Exception as error:
@@ -749,7 +757,7 @@ class DatasetReplicas(RucioController):
     @check_accept_header_wrapper(['application/x-json-stream'])
     def GET(self, scope, name):
         """
-        List dataset replicas replicas.
+        List dataset replicas.
 
         HTTP Success:
             200 OK
@@ -771,7 +779,7 @@ class DatasetReplicas(RucioController):
             if 'deep' in params:
                 deep = params['deep'][0]
         try:
-            for row in list_dataset_replicas(scope=scope, name=name, deep=deep):
+            for row in list_dataset_replicas(scope=scope, name=name, deep=deep, vo=ctx.env.get('vo')):
                 yield dumps(row, cls=APIEncoder) + '\n'
         except RucioException as error:
             raise generate_http_error(500, error.__class__.__name__, error.args[0])
@@ -780,6 +788,53 @@ class DatasetReplicas(RucioController):
             raise InternalError(error)
 
 
+class DatasetReplicasBulk(RucioController):
+
+    @check_accept_header_wrapper(['application/x-json-stream'])
+    def POST(self):
+        """
+        List dataset replicas for multiple DIDs.
+
+        HTTP Success:
+            200 OK
+
+        HTTP Error:
+            400 Bad Request
+            401 Unauthorized
+            406 Not Acceptable
+            500 InternalError
+
+        :returns: A dictionary containing all replicas information.
+        """
+        header('Content-Type', 'application/x-json-stream')
+        json_data = data()
+        try:
+            params = parse_response(json_data)
+            dids = params['dids']
+            didslength = len(dids)
+        except KeyError as error:
+            raise generate_http_error(400, 'KeyError', 'Cannot find mandatory parameter : %s' % str(error))
+        except ValueError:
+            raise generate_http_error(400, 'ValueError', 'Cannot decode json parameter list')
+        except RucioException as error:
+            raise generate_http_error(500, error.__class__.__name__, error.args[0])
+        except Exception as error:
+            print(format_exc())
+            raise InternalError(error)
+        if didslength == 0:
+            raise generate_http_error(400, 'ValueError', 'List of DIDs is empty')
+        try:
+            for row in list_dataset_replicas_bulk(dids=dids, vo=ctx.env.get('vo')):
+                yield dumps(row, cls=APIEncoder) + '\n'
+        except InvalidObject as error:
+            raise generate_http_error(400, 'InvalidObject', 'Cannot validate DIDs: %s' % (str(error)))
+        except RucioException as error:
+            raise generate_http_error(500, error.__class__.__name__, error.args[0])
+        except Exception as error:
+            print(format_exc())
+            raise InternalError(error)
+
+
 class DatasetReplicasVP(RucioController):
 
     @check_accept_header_wrapper(['application/x-json-stream'])
@@ -811,7 +866,7 @@ class DatasetReplicasVP(RucioController):
             if 'deep' in params:
                 deep = params['deep'][0]
         try:
-            for row in list_dataset_replicas_vp(scope=scope, name=name, deep=deep):
+            for row in list_dataset_replicas_vp(scope=scope, name=name, deep=deep, vo=ctx.env.get('vo')):
                 yield dumps(row, cls=APIEncoder) + '\n'
         except RucioException as error:
             raise generate_http_error(500, error.__class__.__name__, error.args[0])
@@ -839,7 +894,7 @@ class ReplicasRSE(RucioController):
         """
         header('Content-Type', 'application/x-json-stream')
         try:
-            for row in list_datasets_per_rse(rse=rse):
+            for row in list_datasets_per_rse(rse=rse, vo=ctx.env.get('vo')):
                 yield dumps(row, cls=APIEncoder) + '\n'
         except RucioException as error:
             raise generate_http_error(500, error.__class__.__name__, error.args[0])
@@ -880,7 +935,7 @@ class BadPFNs(RucioController):
                 state = params['state']
             if 'expires_at' in params and params['expires_at']:
                 expires_at = datetime.strptime(params['expires_at'], "%Y-%m-%dT%H:%M:%S.%f")
-            add_bad_pfns(pfns=pfns, issuer=ctx.env.get('issuer'), state=state, reason=reason, expires_at=expires_at)
+            add_bad_pfns(pfns=pfns, issuer=ctx.env.get('issuer'), state=state, reason=reason, expires_at=expires_at, vo=ctx.env.get('vo'))
         except (ValueError, InvalidType) as error:
             raise generate_http_error(400, 'ValueError', error.args[0])
         except AccessDenied as error:
@@ -922,7 +977,7 @@ class Tombstone(RucioController):
 
         try:
             for replica in replicas:
-                set_tombstone(replica['rse'], replica['scope'], replica['name'], issuer=ctx.env.get('issuer'))
+                set_tombstone(replica['rse'], replica['scope'], replica['name'], issuer=ctx.env.get('issuer'), vo=ctx.env.get('vo'))
         except ReplicaNotFound as error:
             raise generate_http_error(404, 'ReplicaNotFound', error.args[0])
         except RucioException as error:
@@ -940,4 +995,5 @@ class Tombstone(RucioController):
 APP = application(URLS, globals())
 APP.add_processor(loadhook(rucio_loadhook))
 APP.add_processor(unloadhook(rucio_unloadhook))
-application = APP.wsgifunc()
+if __name__ != "rucio.web.rest.replica":
+    application = APP.wsgifunc()
